
# --- libs ---
import pygame as pg


# --- functions ---
def isNaN(s):
	
	try:
		int(s)
		return True
	except ValueError: 
		return False


# --- in range ---
def inRange(comp, val1, val2, d=0):
	
	if int(val1) < int(val2):
		if int(comp) in range(int(val1)-d, int(val2)+d):
			return True
	else:
		if int(comp) in range(int(val2)-d, int(val1)+d):
			return True


def intersection(l,m):
	#global win

	"""
	:find sentence:
	y = ax + b
	a = (y2-y1)/(x2-x1)
	b = y - ax

	:substiotion:
	m into l
	la * x + lb = ma * x + mb
	x = (mb - lb)/(la - ma)

	y = a*res_x + lb

	:special cases:
	if lines is diagonal or horizontal
	this will result in a = 0

	just + 1 or some small number


	"""

	lx = l[0][0] - l[1][0]
	ly = l[0][1] - l[1][1]
	
	mx = m[0][0] - m[1][0]
	my = m[0][1] - m[1][1]

	oddCase = False

	try:
		la = ly/lx
	except ZeroDivisionError:
		lx = l[0][0] - l[1][0] + 0.1
		la = ly/lx
	
	try:
		ma = my/mx
	except ZeroDivisionError:
		mx = m[0][0] - m[1][0] + 0.1
		ma = my/mx

	lb = l[0][1] - la * l[0][0]
	mb = m[0][1] - ma * m[0][0]

	#print('y = '+str(la)+' x+ '+str(lb))
	#print('y = '+str(ma)+' x+ '+str(mb))

	x = (mb - lb)/(la - ma)
	y = la*x+lb

	# - only x and y in lines -
	#pg.draw.rect(win, (0,255,0), [l[0][0], l[0][1], l[1][0]-l[0][0], l[1][1]-l[0][1]], 5)
	if inRange(x, l[0][0], l[1][0], 1) and inRange(y, l[0][1], l[1][1], 1):
		
		if inRange(x, m[0][0], m[1][0], 1) and inRange(y, m[0][1], m[1][1], 1):
			return (int(x), int(y))	
	
	return None


# --- cool borders ---
def sharprect(win,color, rect, thickness=1):
	x,y,w,h = rect

	pg.draw.rect(win, color, [x-int(thickness/2),y-int(thickness/2),thickness,h+thickness])
	pg.draw.rect(win, color, [x-int(thickness/2),y-int(thickness/2),w+thickness,thickness])
	pg.draw.rect(win, color, [x+w-int(thickness/2),y-int(thickness/2),thickness,h+thickness])
	pg.draw.rect(win, color, [x-int(thickness/2),y+h-int(thickness/2),w+thickness,thickness])


# --- trim folder list for macos/darwin ---
def trimfolderlist(l):
	
	if '.DS_Store' in l:
		l.pop(l.index('.DS_Store'))

	return l


# --- wrong place ---
# --- pygame onepress key ---
class Onepress:
	def __init__(self, key, wait=200):
		self.wait = wait
		self.timer = 0
		self.key = key

	def get_pressed(self):
		keys = pg.key.get_pressed()
		if keys[self.key] == 1:
			if self.timer < 0:
				self.timer = int(self.wait)
				return True

		self.timer -= 1
		return False





# --- libs ---
import openpyxl as px
import os

# --- class ---
letters = "ABCDEFGHIJKLMNOPQRSTUVXYZ"

class Database:
	def __init__(self, name, values=[], ids='id'):
		path = os.path.join(os.getcwd(), 'DataPackages', name + '_package')
		if not os.path.exists(path):
			os.makedirs(path)
		
		#print(os.listdir(path))

		self.ids = ids
		self.name = os.path.join(path, 'data_' + name + '.xlsx')
		if 'data_' + name + '.xlsx' not in os.listdir(path):
			workbook = px.Workbook()
			
			self.dataTypes = list(values)
			self.sheetLenghts = 0

			sheet = workbook.active
			for i in range(len(values)):
				sheet[letters[i]+"1"] = values[i]

			workbook.save(filename=self.name)
			count_file = open(os.path.join(path, 'count_file_REST.txt'), 'w')
			count_file.write('0')
			count_file.close()
			self.new = True

		else:
			
			print('retrieving from ' + self.name)
			self.new = False

		self.wb = px.load_workbook(self.name)
		self.sheet = self.wb.active
		
		r = self.get()
		self.dataTypes = r['dataTypes']
		self.sheetLenghts = r['lenght']

		self.count_file = os.path.join(path, 'count_file_REST.txt')


	def append(self, change):


		if type(change) is str:
			print(colors.fg.red+'Please use a list, even for one value'+colors.reset)
			return None
			#assert false
			#raise

		if len(self.get()['dataTypes']) < len(change):
			print(colors.fg.red+'Not enough values'+colors.reset)
			return None

		for i in range(len(change)):
				
			cell = letters[i] + str(self.sheetLenghts+2)
			self.sheet[cell] = change[i]

		self.sheetLenghts += 1

		self.wb.save(filename=self.name)
		
		# - write and close -
		f = open(self.count_file, 'r')
		curId = int(f.read())
		f.close()
		f = open(self.count_file, 'w')
		f.write(str(curId+1))
		f.close()

	def _clear_line(self):
		
		done = False
		while not done:
			done = True

			pl = 'A' + str(place+2) + ':' + letters[len(self.dataTypes)-1] + str(place + self.sheetLenghts - place + 2)
			print(pl)
		
		#ws.move_range("D4:F10", rows=-1)


	def remove(self, id_):
		
		if id_:
			l = self.get()[self.ids]
			
			if id_ in l:
				line = l.index(id_)
				self.sheet.delete_rows(line+2)
				#self._clear_line()
			else:
				print(colors.fg.red+ '"' + id_ + '" not in database'+colors.reset)

		self.wb.save(filename=self.name)


	def get(self):
		
		r_dict = {'dataTypes':[], 'lenght':0}

		# - datatypes -
		l_type = []
		l_values = []
		for i in range(len(letters)):
			r = self.sheet[letters[i] + '1'].value
			if r:
				l_type.append(r)
			

				# - values -
				c = 2
				l_values.append([])
				while True:
					r = self.sheet[letters[i] + str(c)].value

					if r:
						l_values[i].append(r)
					else:
						break

					c += 1

			else:
				break

		l_lenght = []
		for i in l_values:
			l_lenght.append(len(i))


		r_dict = {'dataTypes':l_type, 'values':l_values, 'lenght':max(l_lenght)}
		
		for i in range(len(l_values)):
			r_dict.update({l_type[i]:l_values[i]})

		return r_dict


	def distinct_id(self):
		
		f = open(self.count_file, 'r')
		id_ = f.read()
		f.close()

		while id_ in self.get()[self.ids]:
			id_ = str(int(id_) + 1)

		return id_


	def close(self):
		self.wb.close()




#AUTOMATED FILE
#DO NOT EDIT
#VALTERT
VERSION = "version 0.8.2"

#--- Imports ---
from .screen import BaseScreen
from .screen import Error
from .screen import PopMsg
from .animate.person import PersonAnimation
from .animate.general import Animation
from .database import Database
from .transform import change_color
from .transform import rotate
from .gui.button import Button
from .gui.label import Label
from .gui.gallery import Gallery
from .gui.inp import connectInputs
from .gui.inp import Input
from .pathopen import get_path
from .pathopen import openfilebox
from .engine.movement import PlayerMovement
from .foo import inRange
from .foo import intersection
from .foo import isNaN
from .foo import sharprect
from .foo import trimfolderlist
from .foo import Onepress
from .console.color import color

#--- Assigns ---
BaseScreen = BaseScreen
Error = Error
PopMsg = PopMsg
class animate:
    PersonAnimation = PersonAnimation
    Animation = Animation
Database = Database
change_color = change_color
rotate = rotate
class gui:
    Button = Button
    Label = Label
    Gallery = Gallery
    connectInputs = connectInputs
    Input = Input
get_path = get_path
openfilebox = openfilebox
class engine:
    PlayerMovement = PlayerMovement
inRange = inRange
intersection = intersection
isNaN = isNaN
sharprect = sharprect
trimfolderlist = trimfolderlist
Onepress = Onepress
class console:
    color = color




# Valtert 2020
# Last updated 22. december
# version 0.2
# License: MIT
version = 'version 0.8.2'


# --- lib ---
import os, sys
from inspect import getmembers, isfunction, isclass
from ._all_ import * #for initial test


# --- function ---
def _deleteSubfix(s):
	if s[-3:] == '.py':
		return s[:-3]
	else:
		return s


def _trimFolderList(l):

	if '.DS_Store' in l:
		l.pop(l.index('.DS_Store'))

	if '__init__.py' in l:
		l.pop(l.index('__init__.py'))

	if '__pycache__' in l:
		l.pop(l.index('__pycache__'))

	if '_all_.py' in l:
		l.pop(l.index('_all_.py'))


	return l


def _makeImportText(frm, imp, s, i):
	if len(s) == 0:
		return 'from .' + frm + ' import ' + imp + '\n'
	else:
		return 'from .' + s + '.' + frm + ' import ' + imp + '\n'


def _makeModulesFromFile(modules, s = '', i=''):

	modulesText = ''
	importText = ''

	exec('from .' + s +  ' import ' + modules)

	for func in getmembers(eval(modules), isfunction):
		modulesText += i + func[0] + ' = ' + func[0] + '\n'
		#importText += 'from .' + modules + ' import ' + func[0] + '\n'
		importText += _makeImportText(modules, func[0], s, i)


	for clss in getmembers(eval(modules), isclass):
		modulesText += i + clss[0] + ' = ' + clss[0] + '\n'
		#importText += 'from .' + modules + ' import ' + clss[0] + '\n'
		importText += _makeImportText(modules, clss[0], s, i)

	return modulesText, importText


def makeFile(version):
	
	path = __file__[:-11] + '_all_.py'
	file = open(path, 'w')
	file.write('')
	file.close()
	file = open(path, 'a')

	headFolder = _trimFolderList(os.listdir(__file__[:-11]))

	importText = ''
	modulesText = ''

	for sub in headFolder:
	
		if sub[-3:] == '.py':

			_0, _1 = _makeModulesFromFile(_deleteSubfix(sub))
			modulesText += _0
			importText += _1

		else:

			modulesText += 'class ' + _deleteSubfix(sub) + ':\n'
			for subsub in _trimFolderList(os.listdir(os.path.join(__file__[:-11], sub))):

				if subsub[-3:] == '.py':
					_0, _1 = _makeModulesFromFile(_deleteSubfix(subsub), sub, '    ')
					modulesText += _0
					importText +=  _1

	file.write('\n#AUTOMATED FILE\n#DO NOT EDIT\n#VALTERT\nVERSION = "' + version + '"\n')
	file.write('\n#--- Imports ---\n')
	file.write(importText)
	file.write('\n#--- Assigns ---\n')
	file.write(modulesText)

	file.close()


def test(v):
	
	try:
		if VERSION == v:
			return True
		else:
			return False

	except:
		return False


# --- import ---
if not test(version):
	makeFile(version)
	os.execl(sys.executable, sys.executable, *sys.argv)



# --- libs ---
import pygame as pg
from multiprocessing import Pool
import easygui
import os


"""
pygame blocks main thread for other windows
so a new thread is opened and execute the following lines
"""


# --- function ---
def openfilebox(path):
	r = easygui.fileopenbox(title='amok', default=path)
	return r


def get_path(path):
	
	path = os.path.dirname(path)

	with Pool(1) as p:
		res = p.map(openfilebox, [path])
		print(res)

	return res[0]




"""

pygame.key.start_text_input() -> None

        start handling IME compositions

    Start receiving pygame.TEXTEDITING and pygame.TEXTINPUT events to handle IME.

    A pygame.TEXTEDITING event is received when an IME composition is started or changed. It contains the composition text, length, and editing start position within the composition (attributes text, length, and start, respectively). When the composition is committed (or non-IME input is received), a pygame.TEXTINPUT event is generated.

    Normal pygame.TEXTINPUT events are not dependent on this.

    New in pygame 2.

pygame.key.stop_text_input() -> None

        stop handling IME compositions


"""




# --- libs --- 
from random import randint
import pygame as pg


# --- class ---
currentInput = {}

class Input:
	def __init__(self, x, y, w=200, h=40, color=(255,255,255), font='freesansbold.ttf', fontsize=20, fontcolor=(0,0,0), border=True, erase=False):
		
		self.square = [x, y, w, h]
		self.color = color

		self.border = border
		self.on = False
		self.text = ''

		# - font -
		self.font = pg.font.Font(font, fontsize)
		self.fontcolor = fontcolor
		self.screen = pg.Surface((w-4, h-4))
		self.fontsize = fontsize

		self.id = str(randint(100,99999))
		self.next = None
		self.erase = erase
		self.enter = False


	def draw(self, win):
		
		self.screen.fill(self.color)
		if not self.border:
			pg.draw.rect(win, self.color, self.square)
		else:
			pg.draw.rect(win, self.color, self.square)
			pg.draw.rect(win, self.fontcolor, self.square, 2)

		if len(self.text) > 0:
			render = self.font.render(self.text, True, self.fontcolor)
			self.screen.fill(self.color)
			
			r = render.get_size()[0] + 5

			if r < self.square[2]:
				self.screen.blit(render, (5,5))
			else:
				self.screen.blit(render, (self.square[2]-r,5))

		win.blit(self.screen, (self.square[0]+2, self.square[1]+2))


	def register_event(self, evt):
		
		if self.on and evt:
			
			if evt.key == pg.K_BACKSPACE:
				self.text = self.text[:-1]

				"""
				elif evt.key == pg.K_RETURN:
					print(self.text)
				"""
	
			elif evt.key == pg.K_TAB:
				currentInput[self.next[0]] = (True, self.next[0], self.next[1])
				self.on = False

			elif evt.key == pg.K_RETURN:
				self.enter = True

			else:
				self.text += evt.unicode


	def update(self):
		global currentInput
		

		# - detect -
		ms = pg.mouse.get_pressed()
		pos = pg.mouse.get_pos()
		
		if ms[0] == 1:
			if pos[0] in range(round(self.square[0]), round(self.square[0]+self.square[2])) and pos[1] in range(round(self.square[1]), round(self.square[1]+self.square[3])):
				self.on = True
				if self.erase:
					self.set_text('')
			else:
				self.on = False

		if self.next:
			
			#print(currentInput)
			if currentInput[self.next[0]][2] == self.id:
				if currentInput[self.next[0]][0]:
					currentInput[self.next[0]] = (False, self.next[0], self.id)
					self.on = True

		if self.enter:
			self.enter = False
			return True


	def get_text(self):
		return self.text

	def set_text(self, text):
		self.text = str(text)

	def get_size(self):
		return (self.square[2], self.square[3])

	def get_info(self):
		return {'x':self.square[0], 'y':self.square[1], 'w':self.square[2], 'h':self.square[3],'text':self.text, 'color':self.color, 'font':self.font, 'fontsize':self.fontsize, 'id':self.id, 'size':(self.square[2],self.square[3]), 'border':self.border}


# --- function ---
ident = 0

def connectInputs(inputs): 
	global ident, currentInput

	l = []

	# set next to the next input
	# last gets first
	# so it goes full cirlce

	# in each class when tab is pressed
	# the id and "currentInput" is set to the new one.
	# for each update it is checked

	ident += 1


	for i in range(len(inputs)-1):
		inputs[i].next = (ident,inputs[i+1].id)

	inputs[-1].next = (ident,inputs[0].id)

	currentInput[ident] = (False,-1,-1)




# --- lib ---
import pygame as pg
import fish
from math import *


# --- player movement ---

# good for platformer
# collision with grounds
# in any direction

GRAVITY = 9.82
V0 = 60
TIMEOFFLIGHT = 5
FALLAMP = 0.1

# make an ability to change these properties

# --- class ---
class PlayerMovement:
	def __init__(self, x, y):
		self.x = x
		self.y = y

		self.w = 40
		self.h = 80

		self.top = [(self.x-self.w/2, self.y+self.h/2), (self.x+self.w/2, self.y+self.h/2)]
		self.buttom = [(self.x-self.w/2, self.y-self.h/2), (self.x+self.w/2, self.y-self.h/2)]
		self.left = [(self.x-self.w/2, self.y-self.h/2), (self.x-self.w/2, self.y+self.h/2)]
		self.right = [(self.x+self.w/2, self.y-self.h/2), (self.x+self.w/2, self.y+self.h/2)]

		self.marker_height = 1.8 #1.6
		self.left_marker = [(int(self.x-self.w/2), int(self.y+self.h/self.marker_height)), (int(self.x-self.w/2), int(self.y+self.h/3))]
		self.right_marker = [(int(self.x+self.w/2), int(self.y+self.h/self.marker_height)), (int(self.x+self.w/2), int(self.y+self.h/3))]


		# - move logic -
		self.t = 0
		self.jump = 0
		self.v = 5


	"""
	def _abbreviate_vector_(self, vek, lenght):

		try:
			lengtVec = sqrt(vek[0]**2 + vek[1]**2)
			n = lengtVec / lenght
			return (vek[0]/n, vek[1]/n)
		except ZeroDivisionError:
			return (0,0)
	"""

	def _round_(self, val, con):
		
		if val % con > con/2:
			return val+(con-val%con)
		else:
			return val-val%con


	def _check_collision_(self, lines):
		
		coll_res = []
		
		#player_lines = [self.left_marker, self.right_marker]
		#player_lines = [self.top, self.buttom, self.left, self.right]

		for l in lines:
			
			for pl in [self.left_marker, self.right_marker]:

				res = fish.intersection(pl, l)
				if res:
					coll_res.append(res)

		if len(coll_res) > 0:
			return coll_res

		else:
			return None


	def _collide_with_ground_(self, lines):

		coll_res = self._check_collision_(lines)
		
		#print(int(self.t))
		if int(self.t) not in range(-1, 1):
			if coll_res:
			
				high = coll_res[0]
				for res in coll_res:
					#pg.draw.circle(win, (255,0,0), res, 5) #collision cicles
					if res[1] < high[1]:
						high = list(res)
			
				self.y = high[1] - self.h/2
				self.jump = False


	def movePlayer(self, lines):
	
		# - move -
		keys = pg.key.get_pressed()
		if keys[pg.K_w] == 1:
			if not self.jump:
				self.jump = True
				self.t = 0
		if keys[pg.K_a] == 1:
			self.x -= self.v
		if keys[pg.K_d] == 1:
			self.x += self.v


		# - F(gravity) -
		# v=-g·t+v_0
		
		fall = -GRAVITY * self.t + V0
		self.y -= fall*FALLAMP

		if self.jump:
			self.t += 1/TIMEOFFLIGHT

			#if fall < - V0:
			#	self.jump = False

		else:
			self.t = 12.22


		# - F(normal) -
		self._collide_with_ground_(lines)


		# - update -
		self.top = [
			(int(self.x-self.w/2), int(self.y+self.h/2)), 
			(int(self.x+self.w/2), int(self.y+self.h/2))
		]
		
		self.buttom = [
			(int(self.x-self.w/2), int(self.y-self.h/2)), 
			(int(self.x+self.w/2), int(self.y-self.h/2))
		]
		
		self.left = [
			(int(self.x-self.w/2), int(self.y-self.h/2)), 
			(int(self.x-self.w/2), int(self.y+self.h/2))
		]
		
		self.right = [
			(int(self.x+self.w/2), int(self.y-self.h/2)), 
			(int(self.x+self.w/2), int(self.y+self.h/2))
		]

		self.left_marker = [
			(int(self.x-self.w/2), int(self.y+self.h/self.marker_height)), 
			(int(self.x-self.w/2), int(self.y+self.h/3))
		]
		
		self.right_marker = [
			(int(self.x+self.w/2), int(self.y+self.h/self.marker_height)), 
			(int(self.x+self.w/2), int(self.y+self.h/3))
		]


	def drawHitbox(self, win):

		pg.draw.line(win, (255,0,0), self.top[0], self.top[1], 5)
		pg.draw.line(win, (255,0,0), self.buttom[0], self.buttom[1], 5)
		pg.draw.line(win, (255,0,0), self.left[0], self.left[1], 5)
		pg.draw.line(win, (255,0,0), self.right[0], self.right[1], 5)

		# - center -
		pg.draw.circle(win, (0,255,255), (int(self.x), int(self.y)), 3)
		
		"""
		pg.draw.line(win, (0,255,0), self.left_marker[0], self.left_marker[1], 5)
		pg.draw.line(win, (0,255,0), self.right_marker[0], self.right_marker[1], 5)
		"""



# --- libs --- 
from random import randint
import pygame as pg
import fish


# --- class ---
fonts_initialized = []
fonts = {}
class Label:
	def __init__(self, text, x, y, color=(0,0,255), font='freesansbold.ttf', size=20, base=False):
		self.x = x
		self.y = y

		self.text = str(text)
		self.color = color
		
		try:
			if str(font) + '<>' + str(size) in fonts_initialized:
				self.font = fonts[str(font) + '<>' + str(size)]
			else:
				self.font = pg.font.Font(font, size)
				fonts_initialized.append(str(font) + '<>' + str(size))
				fonts[str(font) + '<>' + str(size)] = self.font
		except Exception as e:
			
			print('\n'+str(e))
			print('\n' + fish.console.color.fg.red + 'Please initialize font in main' + fish.console.color.reset + '\n')
		

		self.fontname = font
		self.render = self.font.render(str(text), True, color)
		self.size = size

		if base:
			self.baseX = int(x)
			self.baseY = int(y)

		self.id = str(randint(100,999))

	def set_text(self, text):
		
		if type(text) is str:
			self.text = text
			self.render = self.font.render(str(text), True, self.color)
		else:
			self.text = str(text)
			self.render = []
			for i in text:
				self.render.append(self.font.render(str(i), True, self.color))


	def draw(self, win):

		if type(self.render) is pg.Surface:
			win.blit(self.render, (self.x, self.y))
		
		else:
			for i in range(len(self.render)):
				win.blit(self.render[i], (self.x, self.y + i*self.size*1.2))

	def get_size(self):
		return self.render.get_size()

	def get_info(self):
		return {'text':self.text, 'color':self.color, 'font':self.fontname, 'id':self.id}

	def update_info(self, iden='', text='', color='', font='', size=''):
		
		try:
			if iden != '':
				self.id = iden
			
			if size != '':
				self.size = int(size)

			if font != '':
				self.fontname = font

			self.font = pg.font.Font(self.fontname, self.size)
			
			if color != '':
				self.color = list(map(int, color.split(',')))

			if text != '':
				self.set_text(text)
			else:
				self.set_text(self.text)

			self.draw(pg.Surface((0,0)))

		except Exception as e:
			print('Error ' + str(e))